@@ -25,7 +25,7 @@ import boto3
 
 from meadowrun.aws_integration.aws_core import _get_default_region_name
 from meadowrun.aws_integration.ec2_instance_allocation import EC2InstanceRegistrar
-from meadowrun.aws_integration.ec2_ssh_keys import ensure_meadowrun_key_pair
+from meadowrun.aws_integration.ec2_ssh_keys import get_meadowrun_ssh_key
 from meadowrun.aws_integration.management_lambdas.ec2_alloc_stub import (
     _EC2_ALLOC_TAG,
     _EC2_ALLOC_TAG_VALUE,
@@ -36,8 +36,8 @@ from meadowrun.run_job_core import RunMapHelper, AllocCloudInstancesInternal
 from meadowrun.shared import pickle_exception
 
 
-_REQUEST_QUEUE_NAME_PREFIX = "meadowrunTaskRequestQueue-"
-_RESULT_QUEUE_NAME_PREFIX = "meadowrunTaskResultQueue-"
+_REQUEST_QUEUE_NAME_PREFIX = "meadowrun-task-request-"
+_RESULT_QUEUE_NAME_PREFIX = "meadowrun-task-result-"
 _QUEUE_NAME_SUFFIX = ".fifo"
 
 _T = TypeVar("_T")
@@ -417,7 +417,7 @@ async def prepare_ec2_run_map(
     if not region_name:
         region_name = await _get_default_region_name()
 
-    pkey = ensure_meadowrun_key_pair(region_name)
+    pkey = get_meadowrun_ssh_key(region_name)
 
     # create SQS queues and add tasks to the request queue
     queues_future = asyncio.create_task(create_queues_and_add_tasks(region_name, tasks))
