@@ -12,12 +12,12 @@ from meadowrun.azure_integration.azure_meadowrun_core import (
     get_default_location,
 )
 from meadowrun.azure_integration.azure_ssh_keys import ensure_meadowrun_key_pair
-from meadowrun.azure_integration.mgmt_functions.azure.azure_exceptions import (
+from meadowrun.azure_integration.mgmt_functions.azure_core.azure_exceptions import (
     ResourceExistsError,
     ResourceModifiedError,
     ResourceNotFoundError,
 )
-from meadowrun.azure_integration.mgmt_functions.azure.azure_storage_api import (
+from meadowrun.azure_integration.mgmt_functions.azure_core.azure_storage_api import (
     StorageAccount,
     azure_table_api,
     azure_table_api_paged,
