@@ -10,10 +10,10 @@ from meadowrun.azure_integration.azure_meadowrun_core import (
     get_default_location,
 )
 from meadowrun.azure_integration.azure_vm_pricing import get_vm_types
-from meadowrun.azure_integration.mgmt_functions.azure.azure_exceptions import (
+from meadowrun.azure_integration.mgmt_functions.azure_core.azure_exceptions import (
     ResourceNotFoundError,
 )
-from meadowrun.azure_integration.mgmt_functions.azure.azure_rest_api import (
+from meadowrun.azure_integration.mgmt_functions.azure_core.azure_rest_api import (
     azure_rest_api,
     azure_rest_api_poll,
     wait_for_poll,
@@ -43,7 +43,7 @@ _MEADOWRUN_COMMUNITY_IMAGE_ID = (
     "/CommunityGalleries/meadowprodeastus-e8b60fd5-8978-467b-a1b0-5b83cbf5393d/Images/"
     "meadowrun"
 )
-_MEADOWRUN_IMAGE_VERSION = "0.1.8"
+_MEADOWRUN_IMAGE_VERSION = "0.1.9"
 
 
 async def _ensure_virtual_network_and_subnet(location: str) -> str:
