@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import abc
 import asyncio
 import dataclasses
 import os.path
@@ -7,6 +8,7 @@ import pickle
 import platform
 import shlex
 import shutil
+import sys
 import tempfile
 import urllib.parse
 import uuid
@@ -38,21 +40,22 @@ from meadowrun.azure_integration.azure_instance_allocation import (
 )
 from meadowrun.azure_integration.azure_ssh_keys import get_meadowrun_vault_name
 from meadowrun.azure_integration.grid_tasks_queue import prepare_azure_vm_run_map
-from meadowrun.azure_integration.mgmt_functions.azure.azure_rest_api import (
+from meadowrun.azure_integration.mgmt_functions.azure_core.azure_rest_api import (
     get_subscription_id_sync,
 )
-from meadowrun.conda import env_export
+from meadowrun.conda import env_export, try_get_current_conda_env
 from meadowrun.config import JOB_ID_VALID_CHARACTERS, MEADOWRUN_INTERPRETER
-from meadowrun.credentials import CredentialsSourceForService
+from meadowrun.credentials import CredentialsSourceForService, CredentialsService
 from meadowrun.deployment import (
     CodeDeployment,
     InterpreterDeployment,
     VersionedCodeDeployment,
     VersionedInterpreterDeployment,
 )
+from meadowrun.docker_controller import get_registry_domain
 from meadowrun.meadowrun_pb2 import (
-    AwsSecret,
-    AzureSecret,
+    AwsSecretProto,
+    AzureSecretProto,
     CodeZipFile,
     ContainerAtDigest,
     ContainerAtTag,
@@ -73,15 +76,337 @@ from meadowrun.meadowrun_pb2 import (
     ServerAvailableInterpreter,
     StringPair,
 )
+from meadowrun.pip_integration import (
+    pip_freeze_without_local_current_interpreter,
+    pip_freeze_without_local_other_interpreter,
+)
 from meadowrun.run_job_core import CloudProviderType, Host, JobCompletion, SshHost
 
 _T = TypeVar("_T")
 _U = TypeVar("_U")
 
 
-# if num_concurrent_tasks isn't specified, by default, launch total_num_tasks *
-# _DEFAULT_CONCURRENT_TASKS_FACTOR workers
-_DEFAULT_CONCURRENT_TASKS_FACTOR = 0.5
+class Secret(abc.ABC):
+    """
+    An abstract class for specifying a secret, e.g. a username/password or an SSH key
+    """
+
+    @abc.abstractmethod
+    def _to_credentials_source(
+        self,
+        service: CredentialsService,
+        service_url: str,
+        credentials_type: Credentials.Type.ValueType,
+    ) -> CredentialsSourceForService:
+        pass
+
+
+@dataclasses.dataclass
+class AwsSecret(Secret):
+    """
+    An AWS secret
+
+    Attributes:
+        secret_name: The name of the secret (also sometimes called the id)
+    """
+
+    secret_name: str
+
+    def _to_credentials_source(
+        self,
+        service: CredentialsService,
+        service_url: str,
+        credentials_type: Credentials.Type.ValueType,
+    ) -> CredentialsSourceForService:
+        return CredentialsSourceForService(
+            service=service,
+            service_url=service_url,
+            source=AwsSecretProto(
+                credentials_type=credentials_type,
+                secret_name=self.secret_name,
+            ),
+        )
+
+
+@dataclasses.dataclass
+class AzureSecret(Secret):
+    """
+    An Azure secret
+
+    Attributes:
+        secret_name: The name of the secret
+        vault_name: The name of the Key Vault that the secret is in. Defaults to None,
+            which implies the Meadowrun-managed Key Vault (mr<last 22 characters of your
+            subscription id>)
+    """
+
+    secret_name: str
+    vault_name: Optional[str] = None
+
+    def _to_credentials_source(
+        self,
+        service: CredentialsService,
+        service_url: str,
+        credentials_type: Credentials.Type.ValueType,
+    ) -> CredentialsSourceForService:
+        if self.vault_name is None:
+            vault_name = get_meadowrun_vault_name(get_subscription_id_sync())
+        else:
+            vault_name = self.vault_name
+
+        return CredentialsSourceForService(
+            service=service,
+            service_url=service_url,
+            source=AzureSecretProto(
+                credentials_type=credentials_type,
+                secret_name=self.secret_name,
+                vault_name=vault_name,
+            ),
+        )
+
+
+class ContainerInterpreterBase(abc.ABC):
+    """An abstract base class for specifying a container as a python interpreter"""
+
+    @abc.abstractmethod
+    def get_interpreter_spec(
+        self,
+    ) -> Union[InterpreterDeployment, VersionedInterpreterDeployment]:
+        pass
+
+    @abc.abstractmethod
+    def _get_repository_name(self) -> str:
+        pass
+
+    @abc.abstractmethod
+    def _get_username_password_secret(self) -> Optional[Secret]:
+        pass
+
+
+@dataclasses.dataclass
+class ContainerInterpreter(ContainerInterpreterBase):
+    """
+    Specifies a container image. The container image must be configured so that running
+    `docker run -it repository_name:tag python` runs the right python interpreter.
+
+    Attributes:
+        repository_name: E.g. `python`, or `<my_azure_registry>.azurecr.io/foo`
+        tag: E.g. `3.9-slim-bullseye` or `latest` (the default)
+        username_password_secret: An AWS or Azure secret that has a username and
+            password for connecting to the container registry (as specified or implied
+            in image_name). Only needed if the image/container registry is private.
+    """
+
+    repository_name: str
+    tag: str = "latest"
+    username_password_secret: Optional[Secret] = None
+
+    def get_interpreter_spec(
+        self,
+    ) -> Union[InterpreterDeployment, VersionedInterpreterDeployment]:
+        return ContainerAtTag(repository=self.repository_name, tag=self.tag)
+
+    def _get_repository_name(self) -> str:
+        return self.repository_name
+
+    def _get_username_password_secret(self) -> Optional[Secret]:
+        return self.username_password_secret
+
+
+@dataclasses.dataclass
+class ContainerAtDigestInterpreter(ContainerInterpreterBase):
+    """
+    Like [ContainerInterpreter][meadowrun.ContainerInterpreter] but specifies a digest
+    instead of a tag. Running `docker run -it repository_name@digest python` must run
+    the right python interpreter.
+
+    Attributes:
+        repository_name: E.g. `python`, or `<my_azure_registry>.azurecr.io/foo`
+        digest: E.g. `sha256:97725c608...`
+        username_password_secret: An AWS or Azure secret that has a username and
+            password for connecting to the container registry (as specified or implied
+            in image_name). Only needed if the image/container registry is private.
+    """
+
+    repository_name: str
+    digest: str
+    username_password_secret: Optional[Secret] = None
+
+    def get_interpreter_spec(
+        self,
+    ) -> Union[InterpreterDeployment, VersionedInterpreterDeployment]:
+        return ContainerAtDigest(repository=self.repository_name, digest=self.digest)
+
+    def _get_repository_name(self) -> str:
+        return self.repository_name
+
+    def _get_username_password_secret(self) -> Optional[Secret]:
+        return self.username_password_secret
+
+
+class InterpreterSpecFile(abc.ABC):
+    """
+    An abstract base class for specifying a Conda environment.yml file or a pip
+    requirements.txt file
+    """
+
+    pass
+
+
+@dataclasses.dataclass
+class CondaEnvironmentYmlFile(InterpreterSpecFile):
+    """
+    Specifies an environment.yml file generated by conda env export.
+
+    Attributes:
+        path_to_yml_file: In the context of mirror_local, this is the path to a file on
+            the local disk. In the context of git_repo, this is a path to a file in the
+            git repo
+    """
+
+    path_to_yml_file: str
+
+
+@dataclasses.dataclass
+class PipRequirementsFile(InterpreterSpecFile):
+    """
+    Specifies a requirements.txt file generated by pip freeze.
+
+    Attributes:
+        path_to_requirements_file: In the context of mirror_local, this is the path to a
+            file on the local disk. In the context of git_repo, this is a path to a file
+            in the git repo
+        python_version: A python version like "3.9" or "3.9.5". The version must be
+            available on docker: https://hub.docker.com/_/python as
+            python-<python_version>-slim-bullseye.
+    """
+
+    path_to_requirements_file: str
+    python_version: str
+
+
+@dataclasses.dataclass
+class PoetryProjectPath(InterpreterSpecFile):
+    """
+    Specifies a poetry project
+
+    Attributes:
+        path_to_project: In the context of mirror_local, this is the path to a folder on
+            the local disk that contains pyproject.toml and poetry.lock. In the context
+            of git_repo, this is a path to a folder in the git repo (use `""` to
+            indicate that pyproject.toml and poetry.lock are at the root of the repo).
+        python_version: A python version like "3.9" or "3.9.5". The version must be
+            available on docker: https://hub.docker.com/_/python as
+            python-<python_version>-slim-bullseye. This python version must be
+            compatible with the requirements in pyproject.toml2
+    """
+
+    path_to_project: str
+    python_version: str
+
+
+class LocalInterpreter(abc.ABC):
+    """
+    An abstract base class for specifying a python interpreter on the local machine
+    """
+
+    @abc.abstractmethod
+    async def get_interpreter_spec(self) -> InterpreterDeployment:
+        pass
+
+
+@dataclasses.dataclass
+class LocalCondaInterpreter(LocalInterpreter):
+    """
+    Specifies a locally installed conda environment
+
+    Attributes:
+        environment_name_or_path: Either the name of a conda environment (e.g.
+            `my_env_name`) or the full path to the folder of a conda environment (e.g.
+            `/home/user/miniconda3/envs/my_env_name`). Will be passed to conda env
+            export
+    """
+
+    environment_name_or_path: str
+
+    async def get_interpreter_spec(self) -> InterpreterDeployment:
+        if platform.system() != "Linux":
+            raise ValueError(
+                "mirror_local from a Conda environment only works from Linux because "
+                "conda environments are not cross-platform."
+            )
+        return EnvironmentSpec(
+            environment_type=EnvironmentType.CONDA,
+            spec=await env_export(self.environment_name_or_path),
+        )
+
+
+@dataclasses.dataclass
+class LocalPipInterpreter(LocalInterpreter):
+    """
+    Specifies a locally available interpreter. It can be a "regular" install of a python
+    interpreter, a virtualenv, or anything based on pip.
+
+    Attributes:
+        path_to_interpreter: The path to the python executable. E.g.
+            `/home/user/my_virtual_env/bin/python`
+        python_version: A python version like "3.9" or "3.9.5". The version must be
+            available on docker: https://hub.docker.com/_/python as
+            python-<python_version>-slim-bullseye.
+    """
+
+    path_to_interpreter: str
+    python_version: str
+
+    async def get_interpreter_spec(self) -> InterpreterDeployment:
+        return EnvironmentSpec(
+            environment_type=EnvironmentType.PIP,
+            # TODO this won't work if the specified environment has editable installs
+            spec=await pip_freeze_without_local_other_interpreter(
+                self.path_to_interpreter
+            ),
+            python_version=self.python_version,
+        )
+
+
+async def _get_current_local_interpreter() -> InterpreterDeployment:
+    # first, check if we are in a conda environment
+    conda_env_spec = await try_get_current_conda_env()
+    if conda_env_spec is not None:
+        if platform.system() != "Linux":
+            raise ValueError(
+                "mirror_local from a Conda environment only works from Linux because "
+                "conda environments are not cross-platform."
+            )
+        print("Mirroring current conda environment")
+        return EnvironmentSpec(
+            environment_type=EnvironmentType.CONDA, spec=conda_env_spec
+        )
+
+    # next, check if we're in a poetry environment. Unfortunately it doesn't seem like
+    # there's a way to detect that we're in a poetry environment unless the current
+    # working directory contains the pyproject.toml and poetry.lock files. If for smoe
+    # reason this isn't the case, we'll fall through to the pip-based case, which will
+    # mostly work for poetry environments.
+    if os.path.isfile("pyproject.toml") and os.path.isfile("poetry.lock"):
+        with open("pyproject.toml") as project_file:
+            project_file_contents = project_file.read()
+        with open("poetry.lock") as lock_file:
+            lock_file_contents = lock_file.read()
+        print("Mirroring current poetry environment")
+        return EnvironmentSpec(
+            environment_type=EnvironmentType.POETRY,
+            spec=project_file_contents,
+            spec_lock=lock_file_contents,
+        )
+
+    # if not, assume this is a pip-based environment
+    print("Mirroring current pip environment")
+    return EnvironmentSpec(
+        environment_type=EnvironmentType.PIP,
+        spec=await pip_freeze_without_local_current_interpreter(),
+        python_version=f"{sys.version_info.major}.{sys.version_info.minor}",
+    )
 
 
 @dataclasses.dataclass(frozen=True)
@@ -100,7 +425,9 @@ class Deployment:
         cls,
         include_sys_path: bool = True,
         additional_paths: Iterable[str] = tuple(),
-        conda_env: Optional[str] = None,
+        interpreter: Union[
+            LocalInterpreter, InterpreterSpecFile, ContainerInterpreterBase, None
+        ] = None,
         environment_variables: Optional[Dict[str, str]] = None,
     ) -> Deployment:
         """A deployment that mirrors the local environment and code.
@@ -110,25 +437,75 @@ class Deployment:
                 (effectively "your local code"), copies them to the remote machines, and
                 add them to sys.path there. Ignores any installed packages.
             additional_paths: local code paths to copy to the remote machine.
-            conda_env: name or full path of locally installed conda environment to
-                use. Defaults to None which means the currently activated env.
+            interpreter: Specifies the environment/interpreter to use. Defaults to None
+                which will detect the currently activated env. Alternatively, you can
+                explicitly specify a locally installed python environment with
+                [LocalCondaInterpreter][meadowrun.LocalCondaInterpreter],
+                [LocalPipInterpreter][meadowrun.LocalPipInterpreter],
+                [CondaEnvironmentYmlFile][meadowrun.CondaEnvironmentYmlFile],
+                [PipRequirementsFile][meadowrun.PipRequirementsFile],
+                [PoetryProjectPath][meadowrun.PoetryProjectPath]
             environment_variables: e.g. `{"PYTHONHASHSEED": "0"}`. These environment
                 variables will be set in the remote environment.
 
         Returns:
             A `Deployment` object that can be passed to the `run_*` functions.
         """
-        if platform.system() != "Linux":
-            raise ValueError(
-                "mirror_local is only supported on Linux "
-                "because Conda environments are not cross-platform."
-            )
-        interpreter = EnvironmentSpec(
-            environment_type=EnvironmentType.CONDA,
-            spec=await env_export(conda_env),
-        )
 
-        code: CodeDeployment = ServerAvailableFolder(code_paths=[])
+        credentials_sources = []
+
+        if interpreter is None:
+            interpreter_spec: Union[
+                InterpreterDeployment, VersionedInterpreterDeployment
+            ] = await _get_current_local_interpreter()
+        elif isinstance(interpreter, CondaEnvironmentYmlFile):
+            with open(interpreter.path_to_yml_file) as f:
+                interpreter_spec = EnvironmentSpec(
+                    environment_type=EnvironmentType.CONDA,
+                    spec=f.read(),
+                )
+        elif isinstance(interpreter, PipRequirementsFile):
+            with open(interpreter.path_to_requirements_file) as f:
+                interpreter_spec = EnvironmentSpec(
+                    environment_type=EnvironmentType.PIP,
+                    spec=f.read(),
+                    python_version=interpreter.python_version,
+                )
+        elif isinstance(interpreter, PoetryProjectPath):
+            with open(
+                os.path.join(interpreter.path_to_project, "pyproject.toml"),
+                "r",
+                encoding="utf-8",
+            ) as spec_file:
+                spec = spec_file.read()
+            with open(
+                os.path.join(interpreter.path_to_project, "poetry.lock"),
+                "r",
+                encoding="utf-8",
+            ) as spec_lock_file:
+                spec_lock = spec_lock_file.read()
+            interpreter_spec = EnvironmentSpec(
+                environment_type=EnvironmentType.POETRY,
+                spec=spec,
+                spec_lock=spec_lock,
+                python_version=interpreter.python_version,
+            )
+        elif isinstance(interpreter, LocalInterpreter):
+            interpreter_spec = await interpreter.get_interpreter_spec()
+        elif isinstance(interpreter, ContainerInterpreterBase):
+            interpreter_spec = interpreter.get_interpreter_spec()
+            username_password_secret = interpreter._get_username_password_secret()
+            if username_password_secret is not None:
+                credentials_sources.append(
+                    username_password_secret._to_credentials_source(
+                        "DOCKER",
+                        get_registry_domain(interpreter._get_repository_name())[0],
+                        Credentials.Type.USERNAME_PASSWORD,
+                    )
+                )
+        else:
+            raise ValueError(f"Unexpected type of interpreter {type(interpreter)}")
+
         # annoyingly, this tmp dir now gets deleted in run_local when the file
         # has been uploaded/unpacked depending on the Host implementation
         tmp_dir = tempfile.mkdtemp()
@@ -139,7 +516,7 @@ class Deployment:
         url = urllib.parse.urlunparse(("file", "", zip_file_path, "", "", ""))
         code = CodeZipFile(url=url, code_paths=zip_paths)
 
-        return cls(interpreter, code, environment_variables, [])
+        return cls(interpreter_spec, code, environment_variables, credentials_sources)
 
     @classmethod
     def git_repo(
@@ -148,10 +525,9 @@ class Deployment:
         branch: Optional[str] = None,
         commit: Optional[str] = None,
         path_to_source: Optional[str] = None,
-        conda_yml_file: Optional[str] = None,
+        interpreter: Union[InterpreterSpecFile, ContainerInterpreterBase, None] = None,
         environment_variables: Optional[Dict[str, str]] = None,
-        ssh_key_aws_secret: Optional[str] = None,
-        ssh_key_azure_secret: Optional[str] = None,
+        ssh_key_secret: Optional[Secret] = None,
     ) -> Deployment:
         """
         A deployment based on a git repo.
@@ -162,20 +538,19 @@ class Deployment:
             commit: can be provided instead of branch to use a specific commit hash,
                 e.g. `"d018b54"`
             path_to_source: e.g. `"src/python"` to use a subdirectory of the repo
-            conda_yml_file: a file (relative to the repo, note that this IGNORES
-                `path_to_source`) file generated by `conda env export`, e.g.
-                `envs/myenv.yml`. This file will be used to generate the environment to
-                run in.
+            interpreter: specifies either a Conda environment.yml or requirements.txt
+                file in the Git repo (relative to the repo, note that this IGNORES
+                `path_to_source`) This file will be used to generate the environment to
+                run in. See
+                [CondaEnvironmentYmlFile][meadowrun.CondaEnvironmentYmlFile],
+                [PipRequirementsFile][meadowrun.PipRequirementsFile], and
+                [PoetryProjectPath][meadowrun.PoetryProjectPath]
             environment_variables: e.g. `{"PYTHONHASHSEED": "0"}`. These environment
                 variables will be set in the remote environment
-            ssh_key_aws_secret: The name of an AWS secret that contains the contents
-                of a private SSH key that has read access to `repo_url`,
-                e.g. `"my_ssh_key"`. See How to use a private git
-                repo for [AWS](/how_to/private_git_repo_aws.md) or
-                [Azure](/how_to/private_git_repo_azure.md)
-            ssh_key_azure_secret: The name of a secret in the meadowrun-generated
-                Azure Vault (which will be named "mr" followed by the last 22
-                letters/numbers of the Azure subscription id).
+            ssh_key_secret: A secret that contains the contents of a private SSH key
+                that has read access to `repo_url`, e.g. `AwsSecret("my_ssh_key")`. See
+                How to use a private git repo for [AWS](/how_to/private_git_repo_aws)
+                or [Azure](/how_to/private_git_repo_azure)
 
         Returns:
             A `Deployment` object that can be passed to the `run_*` functions.
@@ -205,47 +580,51 @@ class Deployment:
                 path_to_source=cast(str, path_to_source),
             )
 
-        if conda_yml_file:
-            interpreter: Optional[InterpreterDeployment] = EnvironmentSpecInCode(
+        credentials_sources = []
+
+        if interpreter is None:
+            interpreter_spec: Union[
+                InterpreterDeployment, VersionedInterpreterDeployment, None
+            ] = None
+        elif isinstance(interpreter, CondaEnvironmentYmlFile):
+            interpreter_spec = EnvironmentSpecInCode(
                 environment_type=EnvironmentType.CONDA,
-                path_to_spec=conda_yml_file,
+                path_to_spec=interpreter.path_to_yml_file,
             )
-        else:
-            interpreter = None
-
-        if ssh_key_aws_secret is not None and ssh_key_azure_secret is not None:
-            # TODO this doesn't seem totally right--we could theoretically specify both
-            # if meadowrun was smart enough to use the right one depending on the
-            # environment
-            raise ValueError("Cannot specify both an AWS secret and an Azure secret")
-
-        if ssh_key_aws_secret is not None:
-            credentials_sources = [
-                CredentialsSourceForService(
-                    service="GIT",
-                    service_url=repo_url,
-                    source=AwsSecret(
-                        credentials_type=Credentials.Type.SSH_KEY,
-                        secret_name=ssh_key_aws_secret,
-                    ),
+        elif isinstance(interpreter, PipRequirementsFile):
+            interpreter_spec = EnvironmentSpecInCode(
+                environment_type=EnvironmentType.PIP,
+                path_to_spec=interpreter.path_to_requirements_file,
+                python_version=interpreter.python_version,
+            )
+        elif isinstance(interpreter, PoetryProjectPath):
+            interpreter_spec = EnvironmentSpecInCode(
+                environment_type=EnvironmentType.POETRY,
+                path_to_spec=interpreter.path_to_project,
+                python_version=interpreter.python_version,
+            )
+        elif isinstance(interpreter, ContainerInterpreterBase):
+            interpreter_spec = interpreter.get_interpreter_spec()
+            username_password_secret = interpreter._get_username_password_secret()
+            if username_password_secret is not None:
+                credentials_sources.append(
+                    username_password_secret._to_credentials_source(
+                        "DOCKER",
+                        get_registry_domain(interpreter._get_repository_name())[0],
+                        Credentials.Type.USERNAME_PASSWORD,
+                    )
                 )
-            ]
-        elif ssh_key_azure_secret is not None:
-            credentials_sources = [
-                CredentialsSourceForService(
-                    service="GIT",
-                    service_url=repo_url,
-                    source=AzureSecret(
-                        credentials_type=Credentials.Type.SSH_KEY,
-                        vault_name=get_meadowrun_vault_name(get_subscription_id_sync()),
-                        secret_name=ssh_key_azure_secret,
-                    ),
-                )
-            ]
         else:
-            credentials_sources = []
+            raise ValueError(f"Unexpected type of interpreter: {type(interpreter)}")
 
-        return cls(interpreter, code, environment_variables, credentials_sources)
+        if ssh_key_secret is not None:
+            credentials_sources.append(
+                ssh_key_secret._to_credentials_source(
+                    "GIT", repo_url, Credentials.Type.SSH_KEY
+                )
+            )
+
+        return cls(interpreter_spec, code, environment_variables, credentials_sources)
 
 
 def _credentials_source_message(
@@ -255,9 +634,9 @@ def _credentials_source_message(
         service=Credentials.Service.Value(credentials_source.service),
         service_url=credentials_source.service_url,
     )
-    if isinstance(credentials_source.source, AwsSecret):
+    if isinstance(credentials_source.source, AwsSecretProto):
         result.aws_secret.CopyFrom(credentials_source.source)
-    elif isinstance(credentials_source.source, AzureSecret):
+    elif isinstance(credentials_source.source, AzureSecretProto):
         result.azure_secret.CopyFrom(credentials_source.source)
     elif isinstance(credentials_source.source, ServerAvailableFile):
         result.server_available_file.CopyFrom(credentials_source.source)
@@ -348,10 +727,16 @@ async def _prepare_code_deployment(
         file_url = urllib.parse.urlparse(code_deploy.url)
         if file_url.scheme != "file":
             raise ValueError(f"Expected file URI: {code_deploy.url}")
-        bucket_name, object_name = await s3.ensure_uploaded(file_url.path)
+        if sys.platform == "win32" and file_url.path.startswith("/"):
+            # on Windows, file:///C:\foo turns into file_url.path = /C:\foo so we need
+            # to remove the forward slash at the beginning
+            file_path = file_url.path[1:]
+        else:
+            file_path = file_url.path
+        bucket_name, object_name = await s3.ensure_uploaded(file_path)
         s3_url = urllib.parse.urlunparse(("s3", bucket_name, object_name, "", "", ""))
         code_deploy.url = s3_url
-        shutil.rmtree(os.path.dirname(file_url.path), ignore_errors=True)
+        shutil.rmtree(os.path.dirname(file_path), ignore_errors=True)
         return code_deploy
     elif target == _DeploymentTarget.AZURE_BLOB_STORAGE:
         raise NotImplementedError("Azure Blob Storage is not implemented yet")
